@@ -1,6 +1,6 @@
 ;;; vc-arch.el --- VC backend for the Arch version-control system
 
-;; Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
 ;; Author:      FSF (see vc.el for full credits)
 ;; Maintainer:  Stefan Monnier <monnier@gnu.org>
@@ -72,7 +72,9 @@
 ;; new functions when we reload this file.
 (put 'Arch 'vc-functions nil)
 
-;;;###autoload (defun vc-arch-registered (file)
+;; XEmacs change: renamed to registered-lite due to 21.4 byte compiler or
+;;                bytecode interpreter issue with redefining vc-arch-registered
+;;;###autoload (defun vc-arch-registered-lite (file)
 ;;;###autoload   (if (vc-find-root file "{arch}/=tagging-method")
 ;;;###autoload       (progn
 ;;;###autoload         (load "vc-arch")
@@ -154,7 +156,7 @@ Only the value `maybe' can be trusted :-(."
 	(with-temp-buffer
 	  (insert-file-contents idfile)
 	  (looking-at ".*[^ \n\t]")
-	  (match-string 0)))
+	  (match-string 0))
       (with-current-buffer (find-file-noselect file)
 	(save-excursion
 	  (goto-char (point-max))
@@ -163,7 +165,7 @@ Only the value `maybe' can be trusted :-(."
 		    (goto-char (point-min))
 		    (re-search-forward vc-arch-tagline-re (+ (point) 1000) t)))
 	      (match-string 1)
-	    (concat "./" (file-relative-name file (vc-arch-root file))))))))
+	    (concat "./" (file-relative-name file (vc-arch-root file)))))))))
 
 (defun vc-arch-tagging-method (file)
   (with-current-buffer
@@ -343,7 +345,6 @@ Return non-nil if FILE is unchanged."
 	    (if (not (re-search-forward "^<<<<<<< " nil t))
 		;; The .rej file is obsolete.
 		(condition-case nil (delete-file rej) (error nil))
-	      ;; XEmacs TODO: we don't have smerge-mode
 	      (smerge-mode 1)
 	      (add-hook 'after-save-hook
 			'vc-arch-delete-rej-if-obsolete nil t)
@@ -409,6 +410,20 @@ Return non-nil if FILE is unchanged."
 
 (defun vc-arch-init-version () nil)
 
+;;; Less obvious implementations.
+
+(defun vc-arch-find-version (file rev buffer)
+  (let ((out (make-temp-file "vc-out")))
+    (unwind-protect
+        (progn
+          (with-temp-buffer
+            (vc-arch-command (current-buffer) 1 nil "file-diffs" file rev)
+            (call-process-region (point-min) (point-max)
+                                 "patch" nil nil nil "-R" "-o" out file))
+          (with-current-buffer buffer
+            (insert-file-contents out)))
+      (delete-file out))))
+
 (provide 'vc-arch)
 
 ;; arch-tag: a35c7c1c-5237-429d-88ef-3d718fd2e704
