@@ -1,7 +1,7 @@
 ;;; vc-hooks.el --- resident support for version-control
 
-;; Copyright (C) 1992, 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2002,
-;;   2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001,
+;;   2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
 ;; Author:     FSF (see vc.el for full credits)
 ;; Maintainer: Andre Spiegel <spiegel@gnu.org>
@@ -53,13 +53,16 @@ BACKEND, use `vc-handled-backends'.")
 (defvar vc-header-alist ())
 (make-obsolete-variable 'vc-header-alist 'vc-BACKEND-header)
 
-(defvar vc-ignore-dir-regexp "\\`\\([\\/][\\/]\\|/net/\\|/afs/\\)\\'"
- "Regexp matching directory names that are not under VC's control.
+(defcustom vc-ignore-dir-regexp "\\`\\([\\/][\\/]\\|/net/\\|/afs/\\)\\'"
+  "Regexp matching directory names that are not under VC's control.
 The default regexp prevents fruitless and time-consuming attempts
 to determine the VC status in directories in which filenames are
-interpreted as hostnames.")
+interpreted as hostnames."
+  :type 'regexp
+  :group 'vc)
 
-(defcustom vc-handled-backends '(RCS CVS SVN SCCS Arch MCVS)
+;; XEmacs change: add GIT
+(defcustom vc-handled-backends '(RCS CVS SVN SCCS Arch MCVS GIT)
   ;; Arch and MCVS come last because they are per-tree rather than per-dir.
   "*List of version control backends for which VC will be used.
 Entries in this list will be tried in order to determine whether a
@@ -319,6 +322,10 @@ non-nil if FILE exists and its contents were successfully inserted."
   "Find the root of a checked out project.
 The function walks up the directory tree from FILE looking for WITNESS.
 If WITNESS if not found, return nil, otherwise return the root."
+  ;; Represent /home/luser/foo as ~/foo so that we don't try to look for
+  ;; witnesses in /home or in /.
+  ;; XEmacs: 2nd arg to `abbreviate-file-name' to match GNU functionality
+  (setq file (abbreviate-file-name file t))
   (let ((root nil))
     (while (not (or root
                    (equal file (setq file (file-name-directory file)))
@@ -359,7 +366,13 @@ backend is tried first."
 	(let ((backend (vc-file-getprop file 'vc-backend)))
 	  (mapcar
 	   (lambda (b)
-	     (and (vc-call-backend b 'registered file)
+	     ;; XEmacs change: autoload-embedded registered functions
+	     ;; renamed to registered-lite due to 21.4 byte compiler or
+	     ;; byte interpreter issues when/if the real -registered is
+	     ;; redefined while executing the fully autoloaded version.
+	     (and (if (fboundp (vc-make-backend-sym b 'registered))
+		      (vc-call-backend b 'registered file)
+		    (vc-call-backend b 'registered-lite file))
 		  (vc-file-setprop file 'vc-backend b)
 		  (throw 'found t)))
 	   (if (or (not backend) (eq backend 'none))
@@ -634,9 +647,10 @@ the user should be returned; if REGEXP is non-nil that means to return
 a regexp for matching all such backup files, regardless of the version."
   (if regexp
       (concat (regexp-quote (file-name-nondirectory file))
-              "\\.~[0-9.]+" (unless manual "\\.") "~")
+              "\\.~.+" (unless manual "\\.") "~")
     (expand-file-name (concat (file-name-nondirectory file)
-                              ".~" (or rev (vc-workfile-version file))
+                              ".~" (subst-char-in-string
+                                    ?/ ?_ (or rev (vc-workfile-version file)))
                               (unless manual ".") "~")
                       (file-name-directory file))))
 
@@ -716,7 +730,8 @@ visiting FILE."
       (setq vc-mode
 	    (let* ((str (if vc-display-status (vc-call mode-line-string file)
 			  (symbol-name backend)))
-		   (test-str "^\\(RCS\\|CVS\\|SVN\\|SCCS\\|Arch\\|MCVS\\)-")
+		   ;; XEmacs change: add GIT
+		   (test-str "^\\(RCS\\|CVS\\|SVN\\|SCCS\\|Arch\\|MCVS\\|GIT\\)-")
 		   (not-locked (string-match test-str str)))
 	      (if (or not-locked (not vc-mode-face))
 		  (concat " " str)
